﻿using Microsoft.AspNetCore.Components;
using MVVMTest2.BusinessModels;
using MVVMTest2.Data;
using MVVMTest2.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace MVVMTest2.ViewModels
{
    // We need to implement IDisposable to unhook events
    public class IndexViewModel : IIndexViewModel, IDisposable
    {
        private readonly DataService DataService;

        public IndexViewModel(DataService DataService)
        {
            this.DataService = DataService;
            // use an API Manager to get data here

            // Set CustomerId for Shopping Cart
            ShoppingCart.CustomerId = 1;

            // Add a few shopping cart items
            AddCartItem(new CartItem()
            {
                Id = 1,
                ProductId = 1,
                Quantity = 3
            });
            AddCartItem(new CartItem()
            {
                Id = 2,
                ProductId = 2,
                Quantity = 1
            });
            AddCartItem(new CartItem()
            {
                Id = 3,
                ProductId = 3,
                Quantity = 2
            });
            UpdateComputedValue();
        }

        public void AddCartItem(CartItem Item)
        {
            // In order to hook the event we need to control 
            // adding and removing cart items
            var model = new CartItemModel(DataService)
            {
                Id = Item.Id,
                ProductId = Item.ProductId,
                Quantity = Item.Quantity
            };
            // Hook the StateChanged event, which is raised from
            // the ProductModel when the Price of a product changes.
            model.Product.StateChanged += Product_StateChanged;

            ShoppingCart.Items.Add(model);
        }

        public void DeleteCartItem(CartItemModel Item)
        {
            // Unhook the StateChanged event before removing from Items
            Item.Product.StateChanged -= Product_StateChanged;
            ShoppingCart.Items.Remove(Item);
        }

        private void Product_StateChanged()
        {
            // When a product price changes, rebuild the computed string
            UpdateComputedValue();
        }

        public CartModel ShoppingCart { get; set; } = new CartModel();

        public void UpdateModels()
        {
            DataService.Products[0].Price += (decimal)1.0;
        }

        public CartItemModel SelectedCartItemModel { get; set; }

        public async Task SelectCartItemModel(ChangeEventArgs args)
        {
            await Task.Delay(0);

            // Set the SelectedCartItemModel based on the item selected in a <select> element
            SelectedCartItemModel = (from x in ShoppingCart.Items
                                     where x.Id == Convert.ToInt32(args.Value.ToString())
                                     select x).FirstOrDefault();
        }

        public string ComputedValue { get; set; }

        /// <summary>
        /// To test the mechanics of updating the UI from a property handler
        /// in a business model, compute a string representing the contents
        /// of the shopping cart
        /// </summary>
        public void UpdateComputedValue()
        {
            var result = "Your cart contains ";
            
            foreach (var item in ShoppingCart.Items)
            {
                result += $"{item.Quantity} of {item.Product.Name} at an item cost of ${item.Product.Price}";
                if (item != ShoppingCart.Items.Last())
                    result += ", ";
                else
                    result += $". The total is ${ShoppingCart.Total}";
            }

            ComputedValue = result;
        }

        /// <summary>
        /// At the end of life, unhook the StateChanged event for all Items
        /// </summary>
        public void Dispose()
        {
            foreach (var item in ShoppingCart.Items)
            {
                item.Product.StateChanged -= Product_StateChanged;
            }
        }
    }
}
